const isQQ = /QQ\/([\d.]+)/i;
const isQzone = /Qzone/i;
const isWx = /MicroMessenger/i;

const qqAdapter = {
  sdkSrc: "//open.mobile.qq.com/sdk/qqapi.js",
  isReady() {
    return !!window.mqq;
  },
  async setShareInfo(config) {
    window.mqq.data.setShareInfo({
      title: config.title,
      desc: config.desc,
      share_url: config.url,
      image_url: config.imageUrl
    });
  }
};

const wxAdapter = {
  sdkSrc: "//res.wx.qq.com/open/js/jweixin-1.6.0.js",
  isReady() {
    return !!window.wx;
  },
  async setShareInfo(config) {
    wx.ready(() => {
      wx.updateAppMessageShareData({
        title: config.title,
        desc: config.desc,
        imgUrl: config.imageUrl,
        link: config.url
      });
    });
  }
};

const platforms = {
  qq: {
    uaRegex: isQQ,
    adapter: qqAdapter
  },
  qzone: {
    uaRegex: isQzone,
    adapter: qqAdapter
  },
  wx: {
    uaRegex: isWx,
    adapter: wxAdapter
  }
};

async function loadJs(src, checkHasScriptFn) {
  let hasScript = false;
  if (typeof checkHasScriptFn === "function")
    hasScript = checkHasScriptFn();
  if (hasScript)
    return await Promise.resolve();
  return await new Promise((resolve, reject) => {
    const script = document.createElement("script");
    script.src = src;
    script.async = true;
    document.body.appendChild(script);
    script.onload = resolve;
    script.onerror = reject;
  });
}

const META_CONFIG = {
  url: {
    itemprop: "url",
    property: "og:url"
  },
  title: {
    itemprop: "name",
    property: "og:title"
  },
  desc: {
    itemprop: "description",
    property: "og:description"
  },
  imageUrl: {
    itemprop: "image",
    property: "og:image"
  }
};
function syncMeta(shareInfo, syncConfig) {
  if (syncConfig === false)
    return;
  Object.keys(META_CONFIG).forEach((key) => {
    const { itemprop, property } = META_CONFIG[key];
    const config = syncConfig?.[key];
    const allowed = !config || typeof config === "undefined" ? true : !!config;
    if (allowed) {
      const el = document.querySelector(`meta[property="${property}"]`);
      const nEl = el ?? document.createElement("meta");
      nEl.setAttribute("itemprop", itemprop);
      nEl.setAttribute("property", property);
      nEl.setAttribute("content", shareInfo[key]);
      if (!el)
        document.head.appendChild(nEl);
    }
  });
}

function trueInfo(info) {
  return !!info && typeof info === "object";
}
const createWebShare = (options) => {
  let globalShareInfo;
  let shareInfo;
  const UA = navigator.userAgent;
  const platformName = Object.keys(platforms).find(
    (name) => platforms[name]?.uaRegex?.test(UA)
  );
  const platformConfig = platformName ? platforms[platformName] : null;
  return {
    setGlobalShareInfo(info) {
      globalShareInfo = info;
    },
    getGlobalShareInfo() {
      return globalShareInfo;
    },
    async setShareInfo(info) {
      shareInfo = info ?? globalShareInfo;
      if (!trueInfo(shareInfo)) {
        return await Promise.reject(
          new Error("必须传入有效的shareInfo，或者设置有效的globalShareInfo")
        );
      }
      const currentShareInfo = shareInfo;
      syncMeta(currentShareInfo, options?.syncMeta);
      if (platformConfig) {
        await loadJs(
          platformConfig.adapter.sdkSrc,
          platformConfig.adapter.isReady
        );
        if (currentShareInfo !== shareInfo)
          return;
        await platformConfig.adapter.setShareInfo(currentShareInfo);
      }
    },
    getShareInfo() {
      return shareInfo;
    }
  };
};
const webShare = createWebShare();

export { createWebShare, webShare as default };
